#include "ina219.h"
#include "port_ini.h"

uint8_t ina219_i2caddr;
uint32_t ina219_calValue;

uint32_t ina219_currentDivider_mA;
uint32_t ina219_powerDivider_mW;


void ina219_gpio_init(void){
	   GPIO_InitTypeDef  GPIO_InitStructure;
	   RCC_APB1PeriphClockCmd(INA219_I2C_GPIO_CLOCK, ENABLE);
	   RCC_APB1PeriphClockCmd(INA219_I2C_PER_CLOCK, ENABLE);
		// I2C2  PB10->SCL and PB11->SDA 
	   GPIO_InitStructure.GPIO_Pin =  INA219_I2C_SCL_PIN | INA219_I2C_SDA_PIN;
     GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	   GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
     GPIO_Init(INA219_I2C_PORT, &GPIO_InitStructure);
     GPIO_PinAFConfig(GPIOB, GPIO_PinSource10, GPIO_AF_I2C2);
     GPIO_PinAFConfig(GPIOB, GPIO_PinSource11, GPIO_AF_I2C2);
    }

void ina219_i2c_init(void){
	   I2C_InitTypeDef  I2C_InitStructure;
     RCC_APB1PeriphClockCmd(INA219_I2C_PER_CLOCK, ENABLE);
     I2C_DeInit(INA219_I2C);
     I2C_InitStructure.I2C_ClockSpeed = INA219_I2C_CLOCK_SPEED;
	   I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
	   I2C_InitStructure.I2C_Mode= I2C_Mode_I2C;
	   I2C_InitStructure.I2C_OwnAddress1 = 0x30;
	   I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
	   I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
		 I2C_Init(INA219_I2C, &I2C_InitStructure);
     I2C_Cmd(INA219_I2C, ENABLE);
     I2C_AcknowledgeConfig(INA219_I2C, ENABLE);
    }




void ina219_writeRegister(uint8_t reg, uint16_t data){
	   while(I2C_GetFlagStatus(INA219_I2C, I2C_FLAG_BUSY));
     I2C_AcknowledgeConfig(INA219_I2C, ENABLE);
     I2C_GenerateSTART(INA219_I2C, ENABLE);
	   while(!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_MODE_SELECT));
	   I2C_Send7bitAddress(INA219_I2C,  INA219_I2C_ADDRESS_DEFAULT, I2C_Direction_Transmitter);
     while (!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
	   I2C_Cmd(INA219_I2C, ENABLE);
     I2C_SendData(INA219_I2C, reg);
	   while (!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
     I2C_SendData(INA219_I2C, (uint8_t) (data >> 8));
	   while (!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
	   I2C_SendData(INA219_I2C, (uint8_t) (data & 0xFF));
	   while (!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
	   I2C_GenerateSTOP(INA219_I2C, ENABLE);
}

void ina219_readRegister(uint8_t reg, uint16_t *data){
	   while(I2C_GetFlagStatus(INA219_I2C, I2C_FLAG_BUSY));
     I2C_AcknowledgeConfig(INA219_I2C, ENABLE);
     I2C_GenerateSTART(INA219_I2C, ENABLE);
	   while(!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_MODE_SELECT));
	   I2C_Send7bitAddress(INA219_I2C,  INA219_I2C_ADDRESS_DEFAULT, I2C_Direction_Transmitter);
	   while (!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
	   I2C_Cmd(INA219_I2C, ENABLE);
     I2C_SendData(INA219_I2C, reg);
	   while (!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
     I2C_GenerateSTART(INA219_I2C, ENABLE);
	   while(!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_MODE_SELECT));
     I2C_Send7bitAddress(INA219_I2C,  INA219_I2C_ADDRESS_DEFAULT, I2C_Direction_Receiver);
     while (!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));
     while(!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED));  /* EV7 */
	   *data = I2C_ReceiveData(INA219_I2C) << 8;
     I2C_AcknowledgeConfig(INA219_I2C, DISABLE);
	   I2C_GenerateSTOP(INA219_I2C, ENABLE);
     while(!I2C_CheckEvent(INA219_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED));  /* EV7 */
	   *data = (uint16_t) I2C_ReceiveData(INA219_I2C) | *data;
   	 I2C_AcknowledgeConfig(INA219_I2C, ENABLE);
   }
void ina219_setCalibration_32V_2A(void){
	   INA219_InitTypeDef INA219_InitStructure;
     INA219_InitStructure.INA219_BusVoltageRange = INA219_BusVoltageRange_32V;
	   INA219_InitStructure.INA219_GainRange = INA219_GainRange_8_320mV;
	   INA219_InitStructure.INA219_BusADCResolution = INA219_BusADCResolution_12Bit;
	   INA219_InitStructure.INA219_ShuntADCResolution = INA219_ShuntADCResolution_12Bit_1S_532uS;
	   INA219_InitStructure.INA219_Mode = INA219_Mode_ShuntAndBusVoltageContinuous;
     ina219_calValue = 4096;
	   ina219_currentDivider_mA = 10;  
	   ina219_powerDivider_mW = 2;     
	   ina219_writeRegister(INA219_REG_CALIBRATION, ina219_calValue);
   	 uint16_t config = INA219_CONFIG_BVOLTAGERANGE_32V |
					INA219_CONFIG_GAIN_8_320MV |
					INA219_CONFIG_BADCRES_12BIT |
					INA219_CONFIG_SADCRES_12BIT_1S_532US |
					INA219_CONFIG_MODE_SANDBVOLT_CONTINUOUS;
	   ina219_writeRegister(INA219_REG_CONFIG, config);
    }

void ina219_configureRegisters(void){
	ina219_currentDivider_mA = 0;
	ina219_powerDivider_mW = 0;
	delay_ms(15);
	ina219_setCalibration_32V_2A();
    }
int16_t ina219_getBusVoltage_raw(void){
	uint16_t value;
	ina219_readRegister(INA219_REG_BUSVOLTAGE, &value);
	return (int16_t)((value >> 3) * 4);
}

int16_t ina219_getShuntVoltage_raw(void){
	uint16_t value;
	ina219_readRegister(INA219_REG_SHUNTVOLTAGE, &value);
	return (int16_t)value;
}

int16_t ina219_getCurrent_raw(void){
	uint16_t value;
	ina219_writeRegister(INA219_REG_CALIBRATION, ina219_calValue);
	ina219_readRegister(INA219_REG_CURRENT, &value);
	return (int16_t)value;
}

float ina219_getShuntVoltage_mV(void){
	int16_t value;
	value = ina219_getShuntVoltage_raw();
	return value * 0.01;
}

float ina219_getBusVoltage_V(void){
	int16_t value = ina219_getBusVoltage_raw();
	return value * 0.001;
}


float ina219_getCurrent_mA(void){
	float valueDec = ina219_getCurrent_raw();
	valueDec /= ina219_currentDivider_mA;
	return valueDec;
}

void ina219_init(void){
	ina219_gpio_init();
	ina219_i2c_init();
	ina219_configureRegisters();
}





