#ifndef __INA219_H
#define __INA219_H

#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_i2c.h"

#define INA219_I2C					    	I2C2
#define INA219_I2C_PORT				  	GPIOB
#define INA219_I2C_PER_CLOCK			RCC_APB1Periph_I2C2
#define INA219_I2C_GPIO_CLOCK		  RCC_AHB1Periph_GPIOB
#define INA219_I2C_SDA_PIN				GPIO_Pin_11
#define INA219_I2C_SCL_PIN				GPIO_Pin_10
#define INA219_I2C_CLOCK_SPEED		100000

#define INA219_I2C_ADDRESS_DEFAULT				   (0x40 << 1)			// A0 = GND, A1 = GND

#define INA219_REG_CONFIG						            (0x00)	
#define INA219_REG_SHUNTVOLTAGE					        (0x01)	
#define INA219_REG_BUSVOLTAGE					          (0x02) 	
#define INA219_REG_POWER						            (0x03)	
#define INA219_REG_CURRENT					            (0x04)	
#define INA219_REG_CALIBRATION					        (0x05)	
#define INA219_CONFIG_BVOLTAGERANGE_32V		      (0x2000)  
#define INA219_CONFIG_GAIN_8_320MV				      (0x1800)  
#define INA219_CONFIG_BADCRES_12BIT				      (0x0400)  
#define INA219_CONFIG_SADCRES_12BIT_1S_532US	  (0x0018)	
#define INA219_CONFIG_MODE_SANDBVOLT_CONTINUOUS	(0x0007)

typedef enum{
	INA219_BusVoltageRange_32V = 0x2000,					
} INA219BusVoltageRange_TypeDef;								

typedef enum{
	INA219_GainRange_8_320mV = 0x1800,						
} INA219GainRange_TypeDef;

typedef enum{
		INA219_BusADCResolution_12Bit = 0x0400,					
}   INA219BusADCResolution_TypeDef;							

typedef enum{
	  INA219_ShuntADCResolution_12Bit_1S_532uS = 0x0018,		
}   INA219ShuntADCResolution_TypeDef;							

typedef enum{
	INA219_Mode_ShuntAndBusVoltageContinuous = 0x0007,	
} INA219Mode_TypeDef;

typedef enum{
	INA219_Mask_Mode = 0x0007,
} INA219Mask_Typedef;

typedef struct{
	INA219BusVoltageRange_TypeDef INA219_BusVoltageRange;
	INA219GainRange_TypeDef INA219_GainRange;
	INA219BusADCResolution_TypeDef INA219_BusADCResolution;
	INA219ShuntADCResolution_TypeDef INA219_ShuntADCResolution;
	INA219Mode_TypeDef INA219_Mode;
}INA219_InitTypeDef;

void ina219_init(void);
float ina219_getBusVoltage_V(void);
float ina219_getCurrent_mA(void);

#endif 
